import controlP5.*;
import peasy.*;
import processing.opengl.*;

PeasyCam cam;
ControlP5 controlP5;
ControlWindow controlWindow;
PMatrix3D currCameraMatrix;
PGraphics3D g3; 

int totalSegments = 10;

PVector[] orbit = new PVector[totalSegments];
Segment[] segments = new Segment[totalSegments];

PVector segmentOffset;

float margins = .1;
CheckBox checkbox;
void setup() {
	size(1200,800,OPENGL); 
	
    cam = new PeasyCam(this, 1000);
    cam.setMinimumDistance(50);
    cam.setMaximumDistance(2000);
    g3 = (PGraphics3D)g;
    controlP5 = new ControlP5(this);                       

    controlWindow = controlP5.addControlWindow("controlP5window",100,100,300,600);
    controlWindow.hideCoordinates();
    controlWindow.setBackground(color(40));

	
    // --------------------------------------------------------------------
    // parse data
    // --------------------------------------------------------------------
    int offset;
    int id = 0;
    float absMax = 100;
    
    offset = 10;
    Controller mySlider;
    mySlider = controlP5.addSlider("margins", 0,.5,.1,    10,offset+id*30, 200,20); id++;
    mySlider.setWindow(controlWindow);
	
	checkbox = controlP5.addCheckBox("checkBox",20,20);  
	checkbox.addItem("draw A",0);
	checkbox.addItem("draw B",0);

	
	
	controlP5.setAutoDraw(false);

    
    // --------------------------------------------------------------------
    // parse data
    // --------------------------------------------------------------------

	for(int i=0;i<totalSegments;i++){
		segments[i] = new Segment();
		if(i==0){
			segments[i].previousSegment = segments[0];
		} else {
			segments[i].previousSegment = segments[i-1];
		}
		segments[i].seed = i;
	}

}

void controlEvent(ControlEvent theEvent) {
	
}
void draw() {
	
	background(0);
	
	// --------------------------------------------------------------------
	// draw base 
	// --------------------------------------------------------------------
	float angleX,angleY;
	int i,j,k;
	float percentage;

	segmentOffset = new PVector(0,0,0);
	// --------------------------------------------------------------------
	// parse data
	// --------------------------------------------------------------------
	
	// custom position of segment 2
	// to see values change easily
	
	
	
	
	segments[1].endRelativeVec.y = .5*(mouseY-height/2);
	segments[1].endRelativeVec.x = .5*(mouseX-width/2);
	segments[1].setControlParams();
	segments[1].resetRelatives();
	
	segments[3].endRelativeVec.y = -.25*(mouseY-height/2);
	segments[3].endRelativeVec.x = -.25*(mouseX-width/2);
	segments[3].setControlParams();
	segments[3].resetRelatives();
	
	
	
	
	for(i=0;i<totalSegments;i++){
		segments[i].setControlParams();
		segments[i].resetRelatives();
		calculateSegmentPosition(i);
	}
	
	// first
	i = 0;
	segments[i].setPreviousOrbit(segments[i].startOrbit);
	segments[i].setNextOrbit(segments[i+1].startOrbit);
	segments[i].parseLerps();
	// middle
	for(i=1;i<(totalSegments-1);i++){
		segments[i].setPreviousOrbit(segments[i-1].endOrbit);
		segments[i].setNextOrbit(segments[i+1].startOrbit);
		segments[i].parseLerps();
	}
	// last
	i = totalSegments-1;
	segments[i].setPreviousOrbit(segments[i-1].endOrbit);
	segments[i].setNextOrbit(segments[i].endOrbit);
	segments[i].parseLerps();
	
	
	
	for(i=0;i<totalSegments;i++){
		segments[i].draw();
		segments[i].drawWireframes();
		segments[i].drawGuides();
	}
	
	gui();
}

void gui() {
	
	drawAxes(400);
    currCameraMatrix = new PMatrix3D(g3.camera);
    camera();
    controlP5.draw();
    g3.camera = currCameraMatrix;
    
    translate(width/2, height/2);

    noFill();
    strokeWeight(1.0);
    stroke(#FFFFFF,25);
    ellipse(0, 0, 200, 200);
    ellipse(0, 0, 180, 180);
}

void calculateSegmentPosition(int i){
	segments[i].createAbsolutes();
	segments[i].updateAbsolutes(segmentOffset);
	segments[i].parse();     
	segmentOffset.add(segments[i].endRelativeVec.get());
}



void keyPressed() {
	if (key == 'r' || key == 'R') {
		for(int i=0;i<totalSegments;i++){
			segments[i].createRelatives();
			segments[i].setControlParams();
			segments[i].resetRelatives();
			segments[i].createAbsolutes();
		}
	}
}

void drawAxes(float radius) {
  noFill();
  strokeWeight(1);
  stroke(255, 0, 0,255);
  line(0, 0, 0, radius, 0, 0);
  stroke(0, 255, 0,255);
  line(0, 0, 0, 0, radius, 0);
  stroke(0, 0, 255,255);
  line(0, 0, 0, 0, 0, radius);

  stroke(#00FF00,51);
  pushMatrix();
  translate(0, radius,0);
  box(10);
  popMatrix();             

  stroke(#FF0000,51);
  pushMatrix();
  translate(radius, 0,0);
  box(10);
  popMatrix(); 

  stroke(#0000FF,51);
  pushMatrix();
  translate(0, 0,radius);
  box(10);
  popMatrix(); 

  stroke(#FFFFFF,51);
  ellipse(0, 0, 200, 200);

  stroke(#FFFFFF,51);
  box(radius*2);
}

